"""Common config module to make the same server accessible in sub-modules
and allow us to add tools"""

from fastmcp import FastMCP

from tagny_mcp_server import __version__

mcp = FastMCP(
    name="Tagny's MCP Servers",
    instructions="web search, URL text fetching, and more tools to enhance locally"
    " served LLMs",
    version=__version__,
)



"""main entry point"""

import typer

from tagny_mcp_server import __version__
from tagny_mcp_server.config import mcp

# import tools to make them available
from tagny_mcp_server.web_access import fetch_page_links, fetch_url_text  # noqa: F401


def main(
    transport: str = "sse",
    host: str = "127.0.0.1",
    port: int = 8000,
    path: str = "/mcp",
    version: bool = False,
):
    """launch mcp server with the same parameters as FastMCP.run() method
    such as

    Args:
        transport: Transport protocol to use ("stdio", "sse", or "streamable-http").
          Defaults to "sse".
        host (str, optional): host address. Defaults to "127.0.0.1".
        port (int, optional): port. Defaults to 8000.
        path (str, optional): path to serve the server at. Defaults to "/mcp".
        version (bool, optional): version of the package.
    """
    if version:
        print(__version__)
        return
    match transport:
        case "stdio":
            mcp.run(transport="stdio")  # Default, so transport argument is optional
        case "sse":
            mcp.run(transport, host=host, port=port)
        case "http":
            mcp.run(transport, host=host, port=port, path=path)


if __name__ == "__main__":
    typer.run(main)



"""A module implementing tools to get data from web pages (e.g. text and URL links)"""

from typing import List

import requests
from bs4 import BeautifulSoup

from tagny_mcp_server.config import mcp


def extract_text_from_html(html_content):
    """
    Extracts plain text from HTML content, removing all HTML tags.

    Args:
        html_content (str): HTML content as string

    Returns:
        str: Plain text content
    """
    soup = BeautifulSoup(html_content, "html.parser")

    # Remove script and style elements
    for script in soup(["script", "style"]):
        script.decompose()

    # Get text and clean it up
    text = soup.get_text()

    # Clean up whitespace
    lines = (line.strip() for line in text.splitlines())
    chunks = (phrase.strip() for line in lines for phrase in line.split("  "))
    text = " ".join(chunk for chunk in chunks if chunk)

    return text


@mcp.tool()
def fetch_url_text(url: str) -> str:
    """Download all visible text from a URL.

    Args:
        url (str): URL to fetch and parse

    Returns:
        str: Plain text content
    """
    try:
        response = requests.get(url, timeout=10)
        response.raise_for_status()
        return extract_text_from_html(response.text)
    except requests.RequestException as e:
        print(f"Error fetching URL: {e}")
        return None


@mcp.tool()
def fetch_page_links(url: str) -> List[str]:
    """Return a list of all links on the page."""
    resp = requests.get(url, timeout=10)
    resp.raise_for_status()
    soup = BeautifulSoup(resp.text, "html.parser")
    return [a["href"] for a in soup.find_all("a", href=True)]



"""a module implementing a fastMCP client to test if the MCP server is started
and if the MCP server's tools are working as expected"""

import pytest
from fastmcp.client import Client

SSE_URL = "http://localhost:8000/sse"
client = Client(SSE_URL)


@pytest.mark.asyncio
async def test_list_tools():
    """test that the list of tools if complete"""
    # Connection is established here
    async with client:
        print(f"Client connected: {client.is_connected()}")

        tools = await client.list_tools()
        print(f"Available tools: {tools}")
        assert len(tools) > 0


@pytest.mark.asyncio
async def test_fetch_url_text():
    """test that the list of tools if complete"""
    # Connection is established here
    async with client:
        print(f"Client connected: {client.is_connected()}")

        tools = await client.list_tools()
        if any(tool.name == "fetch_url_text" for tool in tools):
            result = await client.call_tool(
                "fetch_url_text",
                {
                    "url": "https://apidog.com/blog/fastmcp/",
                },
            )
            print(f"Greet result: {result.content[0].text[:500]} ...")
            assert result is not None
            assert len(result.content[0].text) > 0
